@@ -2,18 +2,10 @@
 #
 # This file is part of the python-crowd library.
 #
-# python-crowd is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# python-crowd is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with python-crowd.  If not, see <http://www.gnu.org/licenses/>.
+# python-crowd is free software released under the BSD License.
+# Please see the LICENSE file included in this distribution for
+# terms of use. This LICENSE is also available at
+# https://github.com/aelse/python-crowd/blob/master/LICENSE
 
 import json
 import requests
@@ -34,15 +26,20 @@ class CrowdServer(object):
 
     Please see the Crowd documentation for information about
     configuring additional applications to talk to Crowd.
+
+    The ``ssl_verify`` parameter controls how and if certificates are verified.
+    If ``True``, the SSL certificate will be verified.
+    A CA_BUNDLE path can also be provided.
     """
 
-    def __init__(self, crowd_url, app_name, app_pass):
+    def __init__(self, crowd_url, app_name, app_pass, ssl_verify=True):
         self.crowd_url = crowd_url
         self.app_name = app_name
         self.app_pass = app_pass
         self.rest_url = crowd_url.rstrip("/") + "/rest/usermanagement/1"
 
         self.session = requests.Session()
+        self.session.verify = ssl_verify
         self.session.auth = requests.auth.HTTPBasicAuth(app_name, app_pass)
         self.session.headers.update({
             "Content-type": "application/json",
@@ -53,7 +50,7 @@ class CrowdServer(object):
         return "Crowd Server at %s" % self.crowd_url
 
     def __repr__(self):
-        return "<CrowdServer('%s', '%s', %s')>" % \
+        return "<CrowdServer('%s', '%s', '%s')>" % \
             (self.crowd_url, self.app_name, self.app_pass)
 
     def _get(self, *args, **kwargs):
@@ -247,6 +244,84 @@ class CrowdServer(object):
         # Otherwise return True
         return True
 
+    def add_user(self, username, **kwargs):
+        """Add a user to the directory
+
+        Args:
+            username: The account username
+            **kwargs: key-value pairs:
+                          password: mandatory
+                          email: mandatory
+                          first_name: optional
+                          last_name: optional
+                          display_name: optional
+                          active: optional (default True)
+
+        Returns:
+            True: Succeeded
+            False: If unsuccessful
+        """
+        # Check that mandatory elements have been provided
+        if 'password' not in kwargs:
+            raise ValueError("missing password")
+        if 'email' not in kwargs:
+            raise ValueError("missing email")
+
+        components = ['username', 'password', 'first_name',
+                      'last_name', 'display_name', 'active']
+        # Populate data with default and mandatory values.
+        # A KeyError means a mandatory value was not provided,
+        # so raise a ValueError indicating bad args.
+        try:
+            data = {
+                    "name": username,
+                    "first-name": username,
+                    "last-name": username,
+                    "display-name": username,
+                    "email": kwargs["email"],
+                    "password": { "value": kwargs["password"] },
+                    "active": True
+                   }
+        except KeyError:
+            return ValueError
+
+        # Remove special case 'password'
+        del(kwargs["password"])
+
+        # Put values from kwargs into data
+        for k, v in kwargs.items():
+            new_k = k.replace("_", "-")
+            if new_k not in data:
+                raise ValueError("invalid argument %s" % k)
+            data[new_k] = v
+
+        response = self._post(self.rest_url + "/user",
+                              data=json.dumps(data))
+
+        if response.status_code == 201:
+            return True
+
+        return False
+
+
+    def get_user(self, username):
+        """Retrieve information about a user
+
+        Returns:
+            dict: User information
+
+            None: If no user or failure occurred
+        """
+
+        response = self._get(self.rest_url + "/user",
+                             params={"username": username,
+                                     "expand": "attributes"})
+
+        if not response.ok:
+            return None
+
+        return response.json()
+
     def get_groups(self, username):
         """Retrieves a list of group names that have <username> as a direct member.
 
